%%%----------------------------------------------------------------------
%%% File    : storage.erl
%%% Author  : Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%% Purpose : 
%%% Created : 3 Jun 2002 by Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%%----------------------------------------------------------------------

-module(storage).
-author('juanjo@juanjo.org').

-behaviour(supervisor).

%% External exports
-export([start_link/1]).

%% supervisor callbacks
-export([init/1]).

%%%----------------------------------------------------------------------
%%% API
%%%----------------------------------------------------------------------
start_link(Drivers) ->
    io:format("storage - start_link function~n"),
    supervisor:start_link(?MODULE, Drivers).

%%%----------------------------------------------------------------------
%%% Callback functions from supervisor
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%----------------------------------------------------------------------
init(Drivers) ->
    io:format("storage - init function~n"),
    io:format("Drivers: ~p~n",[Drivers]), 
    StorageGroup = 
      {storage_group,{storage_group,start_link,
                      [lists:map(fun({DriverName,_,_,_,_,_}) ->
                                    DriverName
                                 end,Drivers)]
		     },
                      permanent,2000,worker,[]},
    io:format("storage - init function2~n"),
    DriverSpecs =
      lists:map(fun({DriverName, DriverType, MOs, MaxConnections, MaxBandwidth, Cost}) ->
			{DriverName,
			 {DriverType,start_link,[DriverName, MOs,MaxConnections,MaxBandwidth, Cost]},
			 permanent,2000,worker,[]}
                end,Drivers),
    io:format("storage - init function3 ~p~n",[[StorageGroup|DriverSpecs]]),
    {ok,{{one_for_one,2,5000}, [StorageGroup|DriverSpecs]}}.

%%%----------------------------------------------------------------------
%%% Internal functions
%%%----------------------------------------------------------------------
