%%%----------------------------------------------------------------------
%%% File    : storage_group.erl
%%% Author  : Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%% Purpose : 
%%% Created : 3 Jun 2002 by Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%%----------------------------------------------------------------------

-module(storage_group).
-author('juanjo@lfcia.org').

%%-compile(export_all).
%%-export([Function/Arity, ...]).

-behaviour(gen_server).

%% External exports
-export([start_link/1]).

%% gen_server callbacks
-export([init/1, handle_call/3]).

-define(DEBUG(F,A),io:format("debug: "++F,A)).
%-define(DEBUG(F,A),ok).

%%%----------------------------------------------------------------------
%%% API
%%%----------------------------------------------------------------------
start_link(StorageDrivers) ->
    ?DEBUG("Starting with ~p ~n", [StorageDrivers]),
    gen_server:start_link({local,?MODULE},?MODULE, StorageDrivers, []).

%%%----------------------------------------------------------------------
%%% Callback functions from gen_server
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%%----------------------------------------------------------------------
init(StorageDrivers) ->
    ?DEBUG("Storage group init~n",[]),
    ?DEBUG("Storage group: Storage Drivers: ~p~n",[StorageDrivers]),
    {ok, {StorageDrivers,0, 0}}.

%%----------------------------------------------------------------------
%% Func: handle_call/3
%% Returns: {reply, Reply, State}          |
%%          {reply, Reply, State, Timeout} |
%%          {noreply, State}               |
%%          {noreply, State, Timeout}      |
%%          {stop, Reason, Reply, State}   | (terminate/2 is called)
%%          {stop, Reason, State}            (terminate/2 is called)
%%----------------------------------------------------------------------
handle_call({lookup,MO,Profile}, From, {StorageDrivers,UsedConnections, UsedBandwidth}) ->
    Options = send_all({lookup,MO,Profile},StorageDrivers,[]),
    ?DEBUG("Storage group - Options: ~p~n",[Options]),
    {reply, {lookupAns,cost(Options,UsedConnections)}, 
     {StorageDrivers,UsedConnections, UsedBandwidth}};

handle_call({play, MO, Bw, Pid}, From, {StorageDrivers, UsedConnections, UsedBandwidth}) ->
    Reply = gen_server:call(Pid, {play,MO, Bw}),
    {reply, Reply, {StorageDrivers, UsedConnections+1, UsedBandwidth+Bw}}.

%%%----------------------------------------------------------------------
%%% Internal functions
%%%----------------------------------------------------------------------

send_all(Message,[], Options) -> Options;
send_all(Message,[Driver|Drivers],Options) -> 
   {lookupAns, NewOptions} = gen_server:call(Driver,Message),
   send_all(Message, Drivers, NewOptions++Options).

cost(Options, Connections) -> 
  Options.
