%%%----------------------------------------------------------------------
%%% File    : storage_sched1.erl
%%% Author  : Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%% Purpose : 
%%% Created : 18 Mar 2002 by Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%%----------------------------------------------------------------------

-module(storage_sched).
-author('juanjo@bulma.lfcia.pri').

%%-compile(export_all).
%%-export([Function/Arity, ...]).

-behaviour(gen_server).

%% External exports
-export([start_link/1]).

%% gen_server callbacks
-export([init/1, handle_call/3]).


%%%----------------------------------------------------------------------
%%% API
%%%----------------------------------------------------------------------
start_link(StorageGroup) ->
    gen_server:start_link({local,?MODULE},?MODULE, StorageGroup, []).

%%%----------------------------------------------------------------------
%%% Callback functions from gen_server
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%%----------------------------------------------------------------------
init(StorageGroup) ->
    io:format("Storage sched init~n"),
    {ok, {StorageGroup,0, 0}}.

%%----------------------------------------------------------------------
%% Func: handle_call/3
%% Returns: {reply, Reply, State}          |
%%          {reply, Reply, State, Timeout} |
%%          {noreply, State}               |
%%          {noreply, State, Timeout}      |
%%          {stop, Reason, Reply, State}   | (terminate/2 is called)
%%          {stop, Reason, State}            (terminate/2 is called)
%%----------------------------------------------------------------------
handle_call({lookup,MO,Profile}, From, {StorageGroup,UsedConnections, UsedBandwidth}) ->
    {lookupAns,Options} = gen_server:call(StorageGroup,{lookup,MO,Profile}),    
    io:format("Storage Sched - Options: ~p~n", [Options]),
    {reply, {lookupAns,cost(Options,UsedConnections)}, 
     {StorageGroup,UsedConnections, UsedBandwidth}};

handle_call({play,MO, Bw, Pid}, From, {StorageGroup, UsedConnections, UsedBandwidth}) ->
    Reply = gen_server:call(StorageGroup, {play,MO, Bw, Pid}),
    {reply, Reply, {StorageGroup, UsedConnections+1, UsedBandwidth+Bw}}.
	

%%%----------------------------------------------------------------------
%%% Internal functions
%%%----------------------------------------------------------------------

cost(Options,Connections) ->
   Options.    
