
%%%----------------------------------------------------------------------
%%% File    : storage_sched1.erl
%%% Author  : Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%% Purpose : 
%%% Created : 18 Mar 2002 by Sanchez Penas Juan Jose <juanjo@bulma.lfcia.pri>
%%%----------------------------------------------------------------------

-module(streaming_sched).
-author('juanjo@bulma.lfcia.pri').

%%-compile(export_all).
%%-export([Function/Arity, ...]).

-behaviour(gen_server).

%% External exports
-export([start_link/1]).

%% gen_server callbacks
-export([init/1, handle_call/3]).


%%%----------------------------------------------------------------------
%%% API
%%%----------------------------------------------------------------------
start_link(StorageSched) ->
    gen_server:start_link({local,?MODULE},?MODULE, StorageSched, []).

%%%----------------------------------------------------------------------
%%% Callback functions from gen_server
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%%----------------------------------------------------------------------
init(StorageSched) ->
    io:format("Streaming Secheduler Init ~n"),
    {ok, {StorageSched, 0, 0}}.

%%----------------------------------------------------------------------
%% Func: handle_call/3
%% Returns: {reply, Reply, State}          |
%%          {reply, Reply, State, Timeout} |
%%          {noreply, State}               |
%%          {noreply, State, Timeout}      |
%%          {stop, Reason, Reply, State}   | (terminate/2 is called)
%%          {stop, Reason, State}            (terminate/2 is called)
%%----------------------------------------------------------------------
handle_call({lookup,MO,Profile}, From, {StorageSched,UsedConnections, UsedBandwidth}) ->
    io:format("Lookup MO Streaming Sched ~n"),
    {lookupAns,Options} = gen_server:call(StorageSched,{lookup,MO,Profile}),    
    io:format("Streaming Sched - Options ~p~n",[Options]),
    {reply, {lookupAns, MO, Profile, cost(Options,UsedConnections)}, 
     {StorageSched,UsedConnections, UsedBandwidth}};

handle_call({play,MO, Bw, Pid}, From, {StorageSched, UsedConnections, UsedBandwidth}) ->
    Reply = gen_server:call(StorageSched, {play,MO, Bw, Pid}),
    {reply, Reply, {StorageSched, UsedConnections+1, UsedBandwidth+Bw}}.
	

%%%----------------------------------------------------------------------
%%% Internal functions
%%%----------------------------------------------------------------------

cost(Options,Connections) ->
   Options.    
